from abc import ABC
import pathlib
from typing import Any, Self

from telegram import Bot, InputFile
from telegram import Message as PTBMessage
import telegram

from ..callback_data import CallbackDataMapping
from ..button_rows import ButtonRows
from ..message import Message          as BaseMessage
from ..message import AudioMessage     as BaseAudioMessage
from ..message import VideoMessage     as BaseVideoMessage
from ..message import DocumentMessage  as BaseDocumentMessage
from ..message import SimpleMessage    as BaseSimpleMessage
from ..message import VideoNoteMessage as BaseVideoNoteMessage
from ..message import PhotoMessage     as BasePhotoMessage

from ..message import SentMessage          as BaseSentMessage
from ..message import SentAudioMessage     as BaseSentAudioMessage
from ..message import SentVideoMessage     as BaseSentVideoMessage
from ..message import SentDocumentMessage  as BaseSentDocumentMessage
from ..message import SentSimpleMessage    as BaseSentSimpleMessage
from ..message import SentVideoNoteMessage as BaseSentVideoNoteMessage
from ..message import SentPhotoMessage     as BaseSentPhotoMessage

class HasButtonRows(ABC):
    def get_reply_markup(self, mapping: CallbackDataMapping):
        if self.button_rows:
            return self.button_rows.to_reply_markup(mapping)
        return None

class AudioMessage(BaseAudioMessage, HasButtonRows):
    def __init__(self, 
            audio: InputFile | bytes | pathlib.Path | telegram.Audio, 
            caption: str, button_rows: ButtonRows = None, parse_mode: str = None):
        super().__init__(caption, button_rows, parse_mode)
        self.audio = audio
    
    async def send(self, user_id: int, bot: Bot, mapping: CallbackDataMapping):
        ptb_message = await bot.send_audio(user_id, self.audio
            , caption = self.caption
            , reply_markup=self.get_reply_markup(mapping)\
            , parse_mode = self.parse_mode)
        return SentSimpleMessage( # TODO: fix
            self.text, self.button_rows, ptb_message)
    
    def __eq__(self, other: "AudioMessage"):
        return self.caption == other.caption and \
            self.audio == other.audio and \
            self.button_rows == other.button_rows
    
    def clone(self):
        button_rows = None
        if self.button_rows:
            button_rows = self.button_rows.clone()
        return AudioMessage(self.audio, self.caption, button_rows, 
            self.parse_mode)

class DocumentMessage(BaseDocumentMessage): ...

class SimpleMessage(BaseSimpleMessage, HasButtonRows):
    async def send(self, user_id: int, bot: Bot, mapping: CallbackDataMapping):
        ptb_message = await bot.send_message(user_id, self.text
            , reply_markup=self.get_reply_markup(mapping)
            , parse_mode=self.parse_mode)
        return SentSimpleMessage(
            self.text, self.button_rows, ptb_message, self.parse_mode)
    
    def __eq__(self, other: "SimpleMessage"):
        return self.text == other.text and \
            self.button_rows == other.button_rows
    
    def clone(self):
        button_rows = None
        if self.button_rows:
            button_rows = self.button_rows.clone()
        return SimpleMessage(self.text, button_rows)

class PhotoMessage(BasePhotoMessage, HasButtonRows): ...

class VideoMessage(BaseVideoMessage, HasButtonRows): ...

class VideoNoteMessage(BaseVideoNoteMessage): ...

class SentAudioMessage(BaseSentAudioMessage, HasButtonRows):
    def __init__(self, 
            audio: str, 
            button_rows: ButtonRows
        , ptb_message: PTBMessage):
        super().__init__(button_rows)
        self.ptb_message = ptb_message 
    
    def change(self, message: SimpleMessage):
        self.text = message.text
        self.button_rows = message.button_rows
    
    async def edit(self, bot: Bot, mapping: CallbackDataMapping):
        orig = self.ptb_message
        reply_markup = self.get_reply_markup(mapping)
        if orig.text == self.text and orig.reply_markup == reply_markup:
            return
        self.ptb_message = await bot.edit_message_text(
            text = self.text,
            reply_markup = reply_markup,
            chat_id=self.ptb_message.chat_id,
            message_id=self.ptb_message.message_id)
    
    async def delete(self, bot: Bot):
        await bot.delete_message(
            chat_id=self.ptb_message.chat_id,
            message_id=self.ptb_message.message_id)
    
    def __eq__(self, other: Self):
        return self.text == other.text and \
            self.button_rows == other.button_rows
    
    def clone(self):
        return SentSimpleMessage(self.text, self.button_rows, self.ptb_message)

    def get_unsent(self):
        return SimpleMessage(
              self.text
            , self.button_rows)

class SentDocumentMessage(BaseSentDocumentMessage, HasButtonRows): ...

# TODO: Добавить поддержку других типов сообщений

class SentSimpleMessage(BaseSentSimpleMessage, HasButtonRows):
    def __init__(self, text: str, button_rows: ButtonRows
        , ptb_message: PTBMessage, parse_mode: str = None):
        super().__init__(text, button_rows, parse_mode)
        self.ptb_message = ptb_message 
    
    def change(self, message: SimpleMessage):
        self.text = message.text
        self.button_rows = message.button_rows
    
    async def edit(self, bot: Bot, mapping: CallbackDataMapping):
        orig = self.ptb_message
        reply_markup = self.get_reply_markup(mapping)
        if orig.text == self.text and orig.reply_markup == reply_markup:
            return
        self.ptb_message = await bot.edit_message_text(
            text = self.text,
            reply_markup = reply_markup,
            parse_mode = self.parse_mode
            chat_id=self.ptb_message.chat_id,
            message_id=self.ptb_message.message_id)
    
    async def delete(self, bot: Bot):
        await bot.delete_message(
            chat_id=self.ptb_message.chat_id,
            message_id=self.ptb_message.message_id)
    
    def __eq__(self, other: Self):
        return self.text == other.text and \
            self.button_rows == other.button_rows
    
    def clone(self):
        return SentSimpleMessage(self.text, self.button_rows, self.ptb_message)

    def get_unsent(self):
        return SimpleMessage(
              self.text
            , self.button_rows)


class SentPhotoMessage(BaseSentPhotoMessage, HasButtonRows): ...

class SentVideoMessage(BaseSentVideoMessage, HasButtonRows): ...

class SentVideoNoteMessage(BaseSentVideoNoteMessage): ...


from abc import ABC, abstractmethod
from typing import Callable, Iterable
from .callback_data import CallbackData
from .message import Message, SentMessage

class HasCallbackData(ABC):
    @abstractmethod
    def __init__(self): ...
    
    def get_callback_data(self):
        result: list[CallbackData] = []
        for message in self.messages:
            result.extend(message.get_callback_data())
        return result

class ReadyScreen(HasCallbackData):
    def __init__(self, *messages: Message):
        self.messages: list[Message] = []
        self.extend(messages)
    
    def extend(self, messages: list[Message]):
        self.messages.extend(messages)
    
    def clone(self) -> "ReadyScreen":
        return ReadyScreen(*[message.clone() for message in self.messages])

class SentScreen(HasCallbackData):
    def __init__(self, *messages: SentMessage):
        self.messages: list[SentMessage] = []
        self.extend(messages)
    
    def extend(self, messages: list[SentMessage]):
        self.messages.extend(messages)
        
    def clone(self) -> "SentScreen":
        return SentScreen(*[message.clone() for message in self.messages])
    
    @abstractmethod
    async def delete(self): ...
    
    @abstractmethod
    def get_unsent(self): ...

class ProtoScreen(ABC):
    def __init__(self, name: str = None):
        self.name = name
        self.messages: list[Message] = []
    
    def append(self, message: Message):
        if not isinstance(message, Message):
            raise ValueError(f"{message=} is not Message")
        self.messages.append(message)
    
    def extend(self, messages: list[Message]):
        for message in messages:
            if not isinstance(message, Message):
                raise ValueError(f"{message=} is not Message")
        self.messages.extend(messages)
    
    @abstractmethod
    def evaluate(self, user_id: int) -> tuple[ReadyScreen, dict[str, CallbackData]]: ...

class StaticScreen(ProtoScreen):
    def __init__(self, name: str, *messages: Message):
        super().__init__(name = name)
        self.extend(messages)
    
    def evaluate(self, _):
        messages = []
        for message in self.messages:
            new_message = message.clone()
            messages.append(new_message)
        return ReadyScreen(*messages)

class DynamicScreen(ProtoScreen):
    def __init__(self, name: str, 
            function: Callable[[int], Iterable[Message]]):
        super().__init__(name)
        self.function = function
    
    def evaluate(self, user_id: int):
        return ReadyScreen(*self.function(user_id))





from typing import Callable
from abc import ABC, abstractmethod

class CallbackData(ABC):
    @abstractmethod
    def clone(self): ...

class RunFunc(CallbackData):
    def __init__(self, function: Callable, **kwargs):
        """Использование:  
            function - Функция для выполнения при нажатии кнопки  
            **kwargs - keyword аргументы функции
        """
        if not isinstance(function, Callable):
            raise ValueError(f"{function=} is not Callable")
        self.function = function
        self.kwargs = kwargs
    
    def clone(self):
        return RunFunc(self.function, **self.kwargs)
    
    def __eq__(self, other: "RunFunc"):
        return isinstance(other, RunFunc) and \
            self.function == other.function and self.kwargs == other.kwargs
        

class GoToScreen(CallbackData):
    def __init__(self, screen_name: str):
        if not isinstance(screen_name, str):
            raise ValueError(f"{screen_name=} is not str")
        self.screen_name = screen_name
    
    def clone(self):
        return GoToScreen(self.screen_name)
    
    def __eq__(self, other: "GoToScreen"):
        return isinstance(other, GoToScreen) and \
            self.screen_name == other.screen_name

class StepBack(CallbackData):
    pass

    def clone(self):
        return StepBack()
    
    def __eq__(self, other: "StepBack"):
        return isinstance(other, StepBack)

# class URLCallbackData(CallbackData):
#     def __init__(self, url: str):
#         self.url = url

class CallbackDataMapping:
    def __init__(self):
        self.items = []
    
    def add(self, callback: CallbackData, uuid: str):
        self.items.append((callback, uuid))
    
    def get_by_callback(self, callback: CallbackData):
        for i_callback, uuid in self.items:
            if callback == i_callback:
                return uuid
        raise KeyError(callback)
    
    def get_by_uuid(self, uuid: str):
        for callback, i_uuid in self.items:
            if uuid == i_uuid:
                return callback
        return None


from abc import abstractmethod
from uuid import uuid4
from .callback_data import CallbackData

def check(obj, condition: bool):
    if not condition:
        raise ValueError(f"{obj=} is wrong")

class Button:
    def __init__(self, text: str, callback_data: CallbackData, url: str = None):
        check( text, isinstance(text, str) and len(text) > 0 )
        check( callback_data, isinstance(callback_data, CallbackData) )
        check( url, not url or url and isinstance(url, str) )
        
        self.text = text
        self.callback_data = callback_data
        self.url = url
    
    def clone(self) -> "Button":
        return Button(self.text, self.callback_data.clone())

    def __eq__(self, other: "Button"):
        return (self.text == other.text and \
            self.callback_data == other.callback_data)

class ButtonRow:
    def __init__(self, *buttons: Button):
        self.buttons: list[Button] = []
        self.extend(buttons)
    
    def extend(self, buttons: list[Button]):
        self.buttons.extend(buttons)
        return self

    def append(self, button: Button):
        self.buttons.append(button)
        return self
    
    def clone(self) -> "ButtonRow":
        return ButtonRow().\
            extend(
                [button.clone() for button in self.buttons]
            )
    def __eq__(self, other: "ButtonRow"):
        return all([
            button1 == button2 
            for button1, button2
            in zip(self.buttons, other.buttons)])

class ButtonRows:
    def __init__(self, *rows: ButtonRow):
        self.rows: list[ButtonRow] = []
        self.extend(rows)
    
    def extend(self, rows: list[ButtonRow]):
        self.rows.extend(rows)
    
    def append(self, row: ButtonRow):
        self.rows.append(row)
    
    def clone(self) -> "ButtonRows":
        return ButtonRows(*[row.clone() for row in self.rows])
    
    def __eq__(self, other: "ButtonRows"):
        return all([row1 == row2
            for row1, row2 in zip(self.rows,other.rows)])
    
    @abstractmethod
    def to_reply_markup(self): ...
    
    def get_callback_data(self):
        result: list[CallbackData] = []
        for row in self.rows:
            for button in row.buttons:
                result.append(button.callback_data)
        return result


from .func_data import FuncData
from .callback_data import CallbackData, CallbackDataMapping
from .screen import SentScreen

class UserData:
    def __init__(self, user_id: int):
        self.user_id = user_id
        self.callback_mapping = CallbackDataMapping()
        self.media_group_id: str = None
        self.after_input: FuncData = None
        self.directory_stack: list[str] = []
        self.screen: SentScreen = None

class UserDataManager:
    def __init__(self):
        self.__users_data: dict[int, UserData] = {}
        self.users_data = self.__users_data
    
    def get(self, user_id: int) -> UserData:
        user_data = self.__users_data.get(user_id)
        if user_data is None:
            user_data = UserData(user_id)
            self.set(user_id, user_data)
        return user_data
    
    def reset(self, user_id: int) -> None:
        self.set(user_id, UserData(user_id))
    
    def set(self, user_id: int, user_data: UserData):
        self.__users_data[user_id] = user_data



from abc import ABC, abstractmethod
from email import message
from typing import Type
from uuid import uuid4

from .callback_data import CallbackDataMapping
from .screen import ProtoScreen, SentScreen
from .message import Message, SentMessage
from .user_data import UserDataManager
from .screen import ReadyScreen

class UserScreen(ABC):
    def __init__(self, user_data: UserDataManager):
        self.user_data = user_data
        self.screen_dict: dict[str, ProtoScreen] = {}
    
    def append_screen(self, screen: ProtoScreen):
        self.screen_dict[screen.name] = screen
    
    def extend_screen(self, screens: list[ProtoScreen]):
        for screen in screens:
            self.append_screen(screen)
    
    @abstractmethod
    async def clear(self, user_id: int, delete_messages: bool): ...
    
    async def set_by_name(self, user_id: int, screen_name: str):
        user_data = self.user_data.get(user_id)
        directory_stack = user_data.directory_stack
        if len(directory_stack)==0 or directory_stack[-1] != screen_name:
            directory_stack.append(screen_name)
        
        screen = self.screen_dict.get(screen_name)
        evaluated_screen = screen.evaluate(user_id)
        
        await self.set(user_id, evaluated_screen)
    
    async def update(self, user_id: int):
        directory_stack = self.user_data.get(user_id).directory_stack
        if len(directory_stack) != 0:
            await self.set_by_name(user_id, directory_stack[-1])
    
    async def step_back(self, user_id: int) -> None:
        directory_stack = self.user_data.get(user_id).directory_stack
        if len(directory_stack) <= 1:
            return
        directory_stack.pop()
        await self.set_by_name(user_id, directory_stack[-1])
    
    def get(self, user_id: int) -> SentScreen | None:
        screen = self.user_data.get(user_id).screen
        if screen is None:
            return None
        return screen.clone()
    
    def _map_callback_data(self, user_id: int, screen: ReadyScreen
            ) -> CallbackDataMapping:
        mapping = CallbackDataMapping()
        callback_data_list = screen.get_callback_data()
        for callback_data in callback_data_list:
            uuid = str(uuid4())
            mapping.add(callback_data, uuid)
        self.user_data.get(user_id).callback_mapping = mapping
        return mapping
        
    @abstractmethod
    async def set(self, user_id: int, new_screen: ReadyScreen):
        ...
    
    @staticmethod
    def calc_screen_difference(screen1: SentScreen, screen2: ReadyScreen):
        messages1 = []
        if screen1:
            messages1 = screen1.messages
        messages2 = screen2.messages
        type_codes = get_type_codes(messages1 + messages2)
        screen1_codes = [type_codes[message.category] 
            for message in messages1]
        screen2_codes = [type_codes[message.category] 
            for message in messages2]
        
        indices_delete, indices_edit, indices_send = calc_abstract_difference(
            screen1_codes, screen2_codes)
        
        messages_delete: list[SentMessage] = [messages1[index]
            for index in indices_delete]
        messages_edit: list[tuple[SentMessage, Message]] = [
            (messages1[from_i],messages2[to_i])
            for from_i, to_i in indices_edit]
        messages_send: list[Message] = [messages2[index]
            for index in indices_send]
        return messages_delete, messages_edit, messages_send

SomeMessage = Message | SentMessage

def get_type_codes(messages: list[SomeMessage]):
    type_codes = set()
    for message in messages:
        type_codes.add(message.category)
    type_codes = list(type_codes)
    type_codes = [(code, i) for i, code in enumerate(type_codes)]
    return dict(type_codes)

def calc_abstract_difference(start: list[int], end: list[int]):
    indices_delete = []
    indices_edit = []
    indices_send = []
    startn = 0
    for j, enum in enumerate(end):
        if startn >= len(start):
            indices_send.append(j)
            continue
        for i, snum in enumerate(start[startn:], start=startn):
            startn += 1
            if enum == snum:
                # (from, to)
                indices_edit.append((i, j))
                break
            else:
                indices_delete.append(i)
        else:
            indices_send = end[j:]
            break
    indices_delete += list(range(startn,len(start)))
    return indices_delete, indices_edit, indices_send


from typing import Callable

class FuncData:
    def __init__(self, function: Callable[[int],None]
        , one_time: bool = True, **kwargs):
        self.function = function
        self.one_time = one_time
        self.kwargs = kwargs


from abc import abstractmethod, ABC
from typing import Callable

from .screen import DynamicScreen
from .func_data import FuncData
from .callback_data import GoToScreen, RunFunc, StepBack, CallbackData
from .user_data import UserDataManager
from .user_screen import UserScreen
from .message import Message

class BotManager(ABC):
    def __init__(self):
        self.system_user_data: UserDataManager = None
        self.screen: UserScreen = None
        
    def config_delete_old_messages(self, user_id: int):
        return True
    
    def build(self):
        user_data = UserDataManager()
        screen = UserScreen(user_data)
        self.system_user_data = user_data
        self.screen = screen
        return self
    
    def get_system_user_data(self, user_id: int):
        return self.system_user_data.get(user_id)

    @abstractmethod
    def get_message_handler(self): ...

    async def _handle_message(self, user_id: int, **kwargs):
        user_data = self.get_system_user_data(user_id)
        if self.config_delete_old_messages(user_id):
            await self.delete_message(**kwargs)
        
        after_input: FuncData = user_data.after_input
        if after_input is None:
            return
        
        await self.screen.clear(user_id, self.config_delete_old_messages(user_id))
        
        if user_data.after_input.one_time:
            user_data.after_input = None
        await after_input.function(user_id=user_id
            , **after_input.kwargs, **kwargs)

    @abstractmethod
    def get_callback_query_handler(self): ...
    
    @abstractmethod
    async def delete_message(self, message): ...
    
    async def mapping_key_error(self, user_id: int): ...
    
    async def _handle_callback_query(self, user_id: int, query_data: str):
        mapping = self.get_system_user_data(user_id).callback_mapping
        data: CallbackData = mapping.get_by_uuid(query_data)
        if data is None:
            await self.mapping_key_error(user_id)
            return
        
        if isinstance(data, GoToScreen):
            await self.screen.set_by_name(user_id, data.screen_name)
        
        elif isinstance(data, StepBack):
            await self.screen.step_back(user_id)
            
        elif isinstance(data, RunFunc):
            await data.function(user_id=user_id, **data.kwargs)
    
    def dynamic_screen(self, name: str):
        def decorator(func: Callable[[int],list[Message]]):
            self.screen.append_screen(DynamicScreen(name, func))
        return decorator


    
    



from uuid import uuid4
from telegram import InlineKeyboardButton, InlineKeyboardMarkup

from ..callback_data import CallbackDataMapping
from ..button_rows import ButtonRows as BaseButtonRows
from ..button_rows import Button as BaseButton
from ..button_rows import ButtonRow as BaseButtonRow


class ButtonRows(BaseButtonRows):
    def to_reply_markup(self, mapping: CallbackDataMapping
            ) -> InlineKeyboardMarkup:
        result = []
        for row in self.rows:
            row_list = []
            for button in row.buttons:
                row_list.append(button.to_inline_button(mapping))
            result.append(row_list)
        return InlineKeyboardMarkup(result)
    
    def clone(self) -> "ButtonRows":
        return ButtonRows(*[row.clone() for row in self.rows])

class Button(BaseButton):
    def to_inline_button(self, mapping: CallbackDataMapping
            ) -> InlineKeyboardButton:
        uuid = mapping.get_by_callback(self.callback_data)
        return InlineKeyboardButton(self.text
            , callback_data = uuid, url = self.url)
    
    def clone(self) -> "Button":
        return Button(self.text, self.callback_data.clone())
        

class ButtonRow(BaseButtonRow): ...


from .bot_manager import BotManager
from .button_rows import ButtonRows, ButtonRow, Button
from .message import   \
      AudioMessage     , SentAudioMessage     \
    , VideoMessage     , SentVideoMessage     \
    , DocumentMessage  , SentDocumentMessage  \
    , SimpleMessage    , SentSimpleMessage    \
    , VideoNoteMessage , SentVideoNoteMessage \
    , PhotoMessage     , SentPhotoMessage     
from .screen import SentScreen
from .user_screen import UserScreen

from ..callback_data import RunFunc, GoToScreen, StepBack
from ..func_data import FuncData
from ..screen import ReadyScreen, StaticScreen, DynamicScreen
from ..message import Message, SentMessage


from abc import ABC, abstractmethod
from typing import Any

from .callback_data import CallbackData
from .button_rows import ButtonRows

class CanBeEdited(ABC):
    @abstractmethod
    async def change(self, message: "Message"): ...
    
    @abstractmethod
    async def edit(self) -> "CanBeEdited": ...

class SentMessage(ABC):
    @abstractmethod
    async def delete(self): ...
    
    @abstractmethod
    def __eq__(self, other: "Message"): ...
    
    @abstractmethod
    def clone(self): ...
    
    @abstractmethod
    def get_unsent(self): ...

class Message(ABC):
    @abstractmethod
    async def send(self, user_id: int) -> SentMessage: ...
    
    @abstractmethod
    def __eq__(self, other: "Message"): ...
    
    @abstractmethod
    def clone(self) -> "Message": ...

    def get_callback_data(self) -> list[CallbackData]:
        if self.button_rows is None:
            return []
        return self.button_rows.get_callback_data()

class AudioMessage(Message):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class DocumentMessage(Message):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class SimpleMessage(Message):
    def __init__(self, text: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.text = text
        self.button_rows = button_rows
        self.category = "simple"

class PhotoMessage(Message):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class VideoMessage(Message):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class VideoNoteMessage(Message):
    def __init__(self, caption: str, button_rows: ButtonRows = None):
        self.caption = caption
        self.button_rows = button_rows
        self.category = "video_note"

class SentAudioMessage(SentMessage, CanBeEdited):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class SentDocumentMessage(SentMessage, CanBeEdited):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"
        

class SentSimpleMessage(SentMessage, CanBeEdited):
    def __init__(self, text: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.text = text
        self.button_rows = button_rows
        self.category = "simple"

class SentPhotoMessage(SentMessage, CanBeEdited):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class SentVideoMessage(SentMessage, CanBeEdited):
    def __init__(self, caption: str, button_rows: ButtonRows = None, 
            parse_mode: str = None):
        self.parse_mode = parse_mode
        self.caption = caption
        self.button_rows = button_rows
        self.category = "media"

class SentVideoNoteMessage(SentMessage):
    def __init__(self, caption: str, button_rows: ButtonRows = None):
        self.caption = caption
        self.button_rows = button_rows
        self.category = "video_note"
